"""Console script for rofipaste."""
import sys
import os
from xdg import BaseDirectory
import click
import click_config_file
from pathlib import Path
from rofipaste import rofipaste

__version__: str = '0.1.3'

config_file_name: str = os.path.join(BaseDirectory.xdg_config_home,
                                     'rofipaste/config')


def createIfNotExist(path):
    """
    Create a directory if it doesn't exists
    """
    Path(path).mkdir(parents=True, exist_ok=True)


@click.command()
@click.option('--version',
              default=False,
              help='Print the current version',
              is_flag=True)
@click.option(
    '--edit-config',
    default=False,
    help='Open your default terminal editor to edit your config file',
    is_flag=True)
@click.option(
    '--edit-entry',
    default=False,
    help=
    'Open your default terminal editor to edit one of your paste entry (or create a new one)',
    is_flag=True)
@click.option(
    '-p',
    '--insert-with-clipboard',
    default=False,
    help=
    'Do not type the characters directly, but copy it to the clipboard, insert it from '
    'there and then restore the clipboard\'s original value',
    is_flag=True)
@click.option(
    '-c',
    '--copy-only',
    default=False,
    help='Only copy the characters to the clipboard but do not insert it',
    is_flag=True)
@click.option('-f',
              '--files',
              default='pastes_folder',
              help='Read pastes from this directory')
@click.option('-r',
              '--prompt',
              default='Rofipaste ❤ ',
              help='Set rofipaste\'s  prompt')
@click.option('--rofi-args',
              default='',
              help='A string of arguments to give to rofi')
@click_config_file.configuration_option(config_file_name=os.path.join(
    BaseDirectory.xdg_config_home, 'rofipaste/config'))
def main(version: bool, edit_config: bool, edit_entry: bool,
         insert_with_clipboard: bool, copy_only: bool, files: str, prompt: str,
         rofi_args: str) -> int:
    """
    RofiPaste is a tool allowing you to copy / paste pieces of codes or other useful texts
    """

    filesPath: str = os.path.join(BaseDirectory.xdg_data_home, 'rofipaste',
                                  files)

    if edit_config:
        click.edit(filename=config_file_name)
        return 0

    if edit_entry:
        filename = click.prompt('Please enter the filename',
                                default="new_entry")
        dirname = os.path.dirname(filename)

        dirname = os.path.join(filesPath, dirname)

        if not os.path.isdir(dirname):
            os.makedirs(dirname)

        click.edit(filename=os.path.join(filesPath, filename))
        return 0

    if version:
        click.echo(f"Current version: {__version__}")
        return 0

    Action = rofipaste.Action
    action = {
        True: Action.TYPE,
        insert_with_clipboard: Action.INSERT_WITH_CLIPBOARD,
        copy_only: Action.COPY_ONLY
    }[True]

    active_window = rofipaste.get_active_window()
    if filesPath[-1] == "/":
        #Removing / at the end to avoid base_folder different from current_folder when going back to top folder
        filesPath = filesPath[:-1]
    print(filesPath)
    createIfNotExist(filesPath)
    base_folder = filesPath
    current_folder = base_folder

    while True:
        folder_content = rofipaste.read_folder_content(current_folder)

        if current_folder != base_folder:
            folder_content = f'{rofipaste.undo_icon} ..\n' + folder_content

        returncode, stdout = rofipaste.open_main_rofi_window(
            rofi_args.split(" "), folder_content, prompt)

        if returncode == 1:
            return 0

        splitted = stdout.rstrip('\n').split(' ')
        icon, path = splitted[0], os.path.join(current_folder,
                                               ' '.join(splitted[1:]))

        if icon == rofipaste.folder_icon:
            current_folder = path

        elif icon == rofipaste.undo_icon:
            current_folder = os.path.dirname(current_folder)

        elif icon == rofipaste.edit_config_icon:
            click.launch(config_file_name)
            return 0

        elif icon in rofipaste.paste_icon_dict.values():
            path = path.rstrip(' (exec)')

            path += '.' + {y: x
                           for x, y in rofipaste.paste_icon_dict.items()}[icon]

            if path[-1] == '.':
                path = path[:-1]

            if 10 <= returncode <= 19:
                # TODO: create shortcuts with 0-9 keys
                #default_handle_recent_character(returncode - 9,  active_window)
                return 0
            else:
                data = rofipaste.fileInterpreter(path)
                if returncode == 0:
                    rofipaste.default_handle(data, action, active_window)
                elif returncode == 20:
                    rofipaste.copy_characters_to_clipboard(data)
                elif returncode == 21:
                    rofipaste.type_characters(data, active_window)
                elif returncode == 22:
                    rofipaste.copy_paste_characters(data, active_window)
                return 0

    return 0


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



"""Main module."""

import os
from subprocess import run, CompletedProcess

from enum import Enum, auto

from typing import List, Tuple, Dict

import click

folder_icon: str = ""
undo_icon: str = ""
edit_config_icon: str = ""
paste_icon_dict: Dict[str, str] = dict(py="",
                                       js="",
                                       java="",
                                       html="",
                                       css="",
                                       c="C",
                                       cpp="C++",
                                       sh="")
paste_icon_dict[''] = ''


class Action(Enum):
    COPY_ONLY = auto()
    INSERT_WITH_CLIPBOARD = auto()
    TYPE = auto()

def read_folder_content(folder_path: str) -> str:
    """read_folder_content.
    
    Read the content of the specified folder and return the found entries

    :param folder_path: Folder's path
    :type folder_path: str
    :rtype: str
    """

    file_entries: str = ''
    exec_entries: str = ''
    dir_entries: str = ''

    for f in os.listdir(folder_path):
        filename = os.path.join(folder_path, f)
        if os.path.isfile(filename):
            with open(filename, 'r') as file_:
                firstline = file_.readline()
            extension = '.'.join(f.split('.')[1:])

            try:
                icon = paste_icon_dict[extension]
                entry_name = f.split('.')[0]
            except:
                icon = paste_icon_dict['']
                entry_name = f

            exec_entries += f'{icon} {entry_name}'

            if firstline[:2] == "#!":
                exec_entries += ' (exec)'

            exec_entries += '\n'
        else:
            dir_entries += f'{folder_icon} {f}\n'

    return (file_entries + exec_entries + dir_entries +
            f"{edit_config_icon} Edit configuration file\n")


def fileInterpreter(path: str) -> str:
    """fileInterpreter

    Interpret the specified file (according to its extension).

    :param path: File's path
    :type path: str
    :rtype: str
    """

    with open(path, 'r') as f:
        content: str = f.read()

    if content[:2] == "#!":
        shebang: str = content.split('\n')[0]
        output: str = run([shebang[2:], path],
                          capture_output=True,
                          encoding='utf-8').stdout
        return output

    return content


def get_active_window() -> str:
    """get_active_window.

    Return the id of the active window

    :rtype: str
    """

    return run(args=['xdotool', 'getactivewindow'],
               capture_output=True,
               encoding='utf-8').stdout[:-1]


def open_main_rofi_window(rofi_args: List[str], characters: str, prompt: str) -> Tuple[int, str]:
    parameters: List[str] = [
        'rofi', '-dmenu', '-markup-rows', '-i', '-multi-select', '-p', prompt,
        '-kb-custom-11', 'Alt+c', '-kb-custom-12', 'Alt+t', '-kb-custom-13',
        'Alt+p', *rofi_args
    ]

    # TODO: Remove "-multi-select" from the parameters ?

    parameters.extend(['-mesg', "Type :edit to edit your config file"])

    rofi: CompletedProcess = run(parameters,
                                 input=characters,
                                 capture_output=True,
                                 encoding='utf-8')

    return rofi.returncode, rofi.stdout


def default_handle(characters: str, action: Action,
                   active_window: str) -> None:
    """default_handle.

    :param characters: The caracters to paste
    :type characters: str
    :param action: The action to perform (paste / type ...)
    :type action: Action
    :param active_window: ID of the active window
    :type active_window: str
    :rtype: None
    """

    if action == Action.COPY_ONLY:
        copy_characters_to_clipboard(characters)

    elif action == Action.INSERT_WITH_CLIPBOARD:
        copy_paste_characters(characters, active_window)

    elif action == Action.TYPE:
        type_characters(characters, active_window)


def copy_characters_to_clipboard(characters: str) -> None:
    """copy_characters_to_clipboard.

    Copy the characters to the clipboard

    :param characters: Characters to copy
    :type characters: str
    :rtype: None
    """

    run(['xsel', '-i', '-b'], input=characters, encoding='utf-8')


def copy_paste_characters(characters: str, active_window: str) -> None:
    """copy_paste_characters.

    Insert characters by copying it and pasting it to the active window

    :param characters: Characters to paste
    :type characters: str
    :param active_window: ID of the active window
    :type active_window: str
    :rtype: None
    """

    old_clipboard_content: str = run(
        args=['xsel', '-o', '-b'], capture_output=True).stdout.decode("utf-8")
    old_primary_content: str = run(args=['xsel', '-o', '-p'],
                                   capture_output=True).stdout.decode("utf-8")

    run(args=['xsel', '-i', '-b'], input=characters, encoding='utf-8')
    run(args=['xsel', '-i', '-p'], input=characters, encoding='utf-8')

    run([
        'xdotool',
        'windowfocus',
        '--sync',
        active_window,
        'key',
        '--clearmodifiers',
        'Shift+Insert',
        'sleep',
        '0.05',
    ])

    run(args=['xsel', '-i', '-b'],
        input=old_clipboard_content,
        encoding='utf-8')
    run(args=['xsel', '-i', '-p'], input=old_primary_content, encoding='utf-8')


def type_characters(characters: str, active_window: str) -> None:
    """type_characters.

    Type the characters in the active window

    :param characters: Characters to type
    :type characters: str
    :param active_window: ID of the active window
    :type active_window: str
    :rtype: None
    """

    run(['xdotool', 'type', '--window', active_window, characters],
        encoding="utf-8")



#!/usr/bin/env python
"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = ['Click>=7.0', 'click_config_file>=0.6.0', 'pyxdg>=0.26']

setup_requirements = [
    'pytest-runner',
]

test_requirements = [
    'pytest>=3',
]

setup(
    author="Tom GOUVILLE",
    author_email='tom.gouville@telecomnancy.net',
    python_requires='>=3.7',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    description="A tool to paste some useful things",
    entry_points={
        'console_scripts': [
            'rofipaste=rofipaste.cli:main',
        ],
    },
    install_requires=requirements,
    license="GNU General Public License v3",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='rofipaste',
    name='rofipaste',
    packages=find_packages(include=['rofipaste', 'rofipaste.*']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/Any0ne22/rofipaste',
    version='0.1.3',
    zip_safe=False,
)
